#include "common.h"
#include "ascii_buf.h"




//returns 1 if rollover occured
//tested and ok
uint8_t bufChangeValue(uint16_t *value, int16_t delta, uint16_t rollover)
{
	if(delta>=0)
	{
		if((*value) < (rollover-delta))
		{
			(*value) += delta;
			return(0);
		}
		else
		{
			(*value) = delta - ( rollover - (*value) );
			return(1);
		}
	}
	else
	{
		delta = -delta;
		if((*value)>=delta)
		{
			(*value) -= delta;
			return(0);
		}
		else
		{
			(*value) = rollover - ( delta - (*value) );
			return(1);
		}
	}
}





void bufRotate8Bits(uint8_t *buf)
{
	uint8_t tempBuf[8];
	uint8_t i,j;
	for(i=0;i<8;i++)
	{
		tempBuf[i]=buf[i];
		buf[i]=0;
	}
	
	for(i=0;i<8;i++)
	{
		for(j=0;j<8;j++)
		{
			if(tempBuf[j] & 0x80) buf[i] |= (1<<j);
			tempBuf[j] <<= 1;
		}
	}
	
	
}







//=================================================================================================
//								STRING UTILS
//=================================================================================================

//compares two strings in buf1 and buf2, ignores case
//user must specify len input parameter - number of compared chars
//returns 1 when buf1 contains the same string as buf2
//returns 0 if bufs are different
uint8_t bufCompareString(Tbuf *buf1, Tbuf *buf2, uint8_t len)
{
	uint8_t i, temp1, temp2;
	for(i=0;i<len;i++)
	{
		temp1 = buf1[i];
		temp2 = buf2[i];
		//convert to upper case
		if( (temp1>='a')&&(temp1<='z') ) temp1 &= ~(1<<5);
		if( (temp2>='a')&&(temp2<='z') ) temp2 &= ~(1<<5);
		//debug_msg("copare string: temp1="); debug8_t(temp1); debug_txt(" temp2="); debug8_t(temp2);
		if(temp1!=temp2) return(0);
	}
	return(1);
}

//compares two sequencez from buf1 und buf2
//number of chars must be provided in len
//returns 1 if buf1 and buf2 contents are the same
inline uint8_t bufCompareSequence(Tbuf *buf1, Tbuf *buf2, uint8_t len)
{
	uint8_t i;
	for(i=0;i<len;i++)
		if(buf1[i]!=buf2[i]) return(0);
	return(1);
}


//returns index of buffer containing specified value
//searches until reaches char=0
uint16_t bufFindChar(Tbuf *buf, uint8_t value)
{
	uint16_t bufindex=0;
	const uint16_t limit = 10000;
	for(;;)
	{
		if(((*buf)==value)||( *buf==0 )) return(bufindex);
		bufindex++;
		buf++;
		if(bufindex>limit) return(limit);
	}
	return(0);
}


//returns index of buffer containing specified value
uint16_t bufFindValue(Tbuf *buf, uint8_t value, uint16_t limit)
{
	uint16_t bufindex=0;
	for(;;)
	{
		if( (*buf)==value ) return(bufindex);
		bufindex++;
		buf++;
		if(bufindex>limit) return(limit);
	}
	return(0);
}


//works similar to CTRL+F. returns index to the beginning of requested sequence
//buf: input buffer
//reference: sequence we are looking for
//buflen: length of buffer (where to stop)
//seqLen: length of reference sequence
//returns: index in buffer, where the sequence was found. returns 0xFFFF if no match
uint16_t bufFindSequence(Tbuf *buf, uint16_t bufLen, Tbuf *reference, uint8_t seqLen)
{
	uint16_t idx = 0;
	while(idx < bufLen)
	{
		//search for the first byte
		idx += bufFindValue(&buf[idx],reference[0],bufLen);
		//debug_value("bufFindSequence, idx = ",idx);
		if( bufCompareSequence(&buf[idx],reference,seqLen) )	//if sequence if found
			return(idx);
		else
			idx++;
	}
	return(0xFFFF);
}


//a substitute for strcpy but works with Tbuf
void bufStrcpy(Tbuf *bufNew, Tbuf *bufOld)
{
//	strcpy((char*)bufNew,(char*)bufOld);
	while((*bufOld)!=0)
	{
		*bufNew=*bufOld;
		bufNew++;
		bufOld++;
	};
	*bufNew=*bufOld;
}

void bufCopy(Tbuf *bufNew, Tbuf *bufOld, uint16_t len)
{
	while(len)
	{
		*bufNew=*bufOld;
		bufNew++;
		bufOld++;
		len--;
	};
}


//a substitute for strlen
uint16_t bufStrlen(Tbuf *buf)
{
	uint16_t len=0;
	while((*buf)!=0)
	{
		buf++;
		len++;
	}
	return(len);
}


//concatenate 2 strings: adds src at the end of dest
uint16_t bufStrcat(Tbuf *dest, Tbuf *src)
{
	uint16_t len = bufStrlen(dest);
	Tbuf *ptr;
	ptr = &dest[len];
	bufStrcpy(ptr,src);
	return(bufStrlen(dest));
}




//=================================================================================================
//								PRIVATE UTILS
//=================================================================================================


//returns integer value of hexadecimal ascii char
uint8_t bufHexToDec(uint8_t hexdigit)
{
	if(hexdigit<0x3A) //ascii digit
	{
		return(hexdigit-'0');
	}
	else //A-F or a-f
	{
		hexdigit &= ~(1<<5); //convert to A-F
		return(hexdigit-'a'+10);
	}
}


//returns ascii char 0..9, A-F of input value (input val must be 0..15)
uint8_t bufDecToHex(uint8_t val)
{
	if(val>9) return(val-10+'A'); else return(val+'0');
}


//=================================================================================================
//								PUT
//=================================================================================================

//puts uint8_t hex coded into buffer
void bufPutHex8_t(Tbuf* buf, uint8_t val)
{
	*buf=bufDecToHex(val>>4);
	buf++;
	*buf=bufDecToHex(val&0x0F);
}


//puts uint16_t hex coded into buffer
void bufPutHex16_t(Tbuf* buf, uint16_t val)
{
	bufPutHex8_t(buf,val>>8);
	buf+=2;
	bufPutHex8_t(buf,val&0xFF);
}


//puts uint16_t hex coded into buffer
void bufPutHex32_t(Tbuf* buf, uint32_t val)
{
	bufPutHex8_t(buf,val>>24);
	buf+=2;
	bufPutHex8_t(buf,val>>16);
	buf+=2;
	bufPutHex8_t(buf,val>>8);
	buf+=2;
	bufPutHex8_t(buf,val>>0);
}



//prints 1, 2 or 3 decimal digits in buffer
void bufPutDec8_t(Tbuf* buf, uint8_t val, uint8_t digits)
{
	if(digits>=3) {*buf=val/100+'0'; buf++;}
	val=val%100;
	if(digits>=2) {*buf=val/10+'0'; buf++;}
	val=val%10;
	*buf=val+'0';
}


//prints 1, 2, 3, 4, or 5 decimal digits in a buffer
void bufPutDec16_t(Tbuf* buf, uint16_t val, uint8_t digits)
{
	uint16_t temp1=0;
	if(digits>=5) {*buf = (val/10000+0x30); buf++;}
	temp1 = (val%10000);
	if(digits>=4) {*buf = (temp1/1000+0x30); buf++;}
	temp1 = temp1%1000;
	if(digits>=3) {*buf = (temp1/100+0x30); buf++;}
	temp1 = temp1%100;
	if(digits>=2) {*buf = (temp1/10+0x30); buf++;}
	temp1 = temp1%10;
	*buf = (temp1+0x30);
}


//prints 1, 2, 3, 4, or 5 decimal digits in the buffer
void bufPutDec32_t(Tbuf* buf, uint32_t val, uint8_t digits)
{
	uint32_t divisor = 1000000000;
	uint32_t temp1=val;
	//debug_msg("entering bufPutDec32_t...");
	uint8_t i;
	for(i=10;i>1;--i)
	{
		//debug_msg("i="); debug8_t(i); 
		if(digits>=i)
		{
			*buf = (Tbuf)(temp1/divisor+0x30);
			//debug_txt(" *buf="); debug8_t(*buf);
 			buf++;
		}
		//else debug_msg("zonk!");
		temp1 = (temp1%divisor);
		//debug_txt(" temp1="); debug32_t(temp1);
		divisor = divisor / 10;
	}
	*buf = temp1+0x30;
	//debug_msg("before exit, buf:");
	//debug_dump(backup,10);
}


//puts decimal number into the buffer
//digit count depends on value
uint8_t bufPutDecVar(Tbuf* buf, uint16_t val)
{
	uint16_t temp1=0;
	uint8_t len=1;
	if(val>=10000) {*buf = (val/10000+0x30); buf++; len++;}
	temp1 = (val%10000);
	if(val>=1000) {*buf = (temp1/1000+0x30); buf++; len++;}
	temp1 = temp1%1000;
	if(val>=100) {*buf = (temp1/100+0x30); buf++; len++;}
	temp1 = temp1%100;
	if(val>=10) {*buf = (temp1/10+0x30); buf++; len++;}
	temp1 = temp1%10;
	*buf = (temp1+0x30);
	return(len);
}



//=================================================================================================
//								GET
//=================================================================================================


//reads decimal value of 1 to 10 digits length. Length is computed automatically.
//returns zero if number is longer than 5 digitz
uint32_t bufGetDec(Tbuf* buf)
{
	uint8_t len;
	len=bufGetNumberLengthDec(buf);
	//debug_value("bufGetDec, len=",len);
	if(len>10) return(0);
	return(bufGetDec32_t(buf,len));
}


//reads hex value of 2 or 4 digits length. Length is computed automatically.
//returns zero if number is not (2 or 4) digitz
uint16_t bufGetHex(Tbuf* buf)
{
	uint8_t len;
	len=bufGetNumberLengthHex(buf);
	if(len==2)
		return(bufGetHex8_t(buf));
	else if(len==4)
		return(bufGetHex16_t(buf));
	else
		return(0);
}


//finds and returns index to the first decimal ascii digit in the buffer
uint16_t bufFindDigitDec(Tbuf* buf)
{
	uint16_t retval=0;
	uint8_t temp;
	for(;;)
	{
		temp=*buf;
		if(((temp>='0')&&(temp<='9'))) break;
		retval++;
		buf++;
	}
	return(retval);
}


//finds and returns index to the first hex ascii digit in the buffer
uint16_t bufFindDigitHex(Tbuf* buf)
{
	uint16_t retval=0;
	uint8_t temp;
	for(;;)
	{
		temp=*buf;
		if((((temp>='0')&&(temp<='9'))||((temp>='a')&&(temp<='f'))||((temp>='A')&&(temp<='F')))) break;
		retval++;
		buf++;
	}
	return(retval);
}


//returns length (in digits) of number that starts at buf location. Works for Hex
uint16_t bufGetNumberLengthHex(Tbuf* buf)
{
	uint16_t retval=0;
	uint8_t temp;
	for(;;)
	{
		temp=*buf;
		if(!(((temp>='0')&&(temp<='9'))||((temp>='a')&&(temp<='f'))||((temp>='A')&&(temp<='F')))) break;
		retval++;
		buf++;
	}
	return(retval);
}


//returns length (in digits) of number that starts at buf location. Works for Dec
uint16_t bufGetNumberLengthDec(Tbuf* buf)
{
	uint16_t retval=0;
	Tbuf temp;
	for(;;)
	{
		temp=*buf;
		if(!((temp>='0')&&(temp<='9'))) break;
		retval++;
		buf++;
	}
	return(retval);
}


//reads decimal number from ascii bufer into integer. Specify length of number in digits parameter.
uint16_t bufGetDec16_t(Tbuf* buf, uint8_t digits)
{
	uint16_t retval=0;
	if(digits>=5) {retval += (*buf-'0')*10000; buf++;}
	if(digits>=4) {retval += (*buf-'0')*1000; buf++;}
	if(digits>=3) {retval += (*buf-'0')*100; buf++;}
	if(digits>=2) {retval += (*buf-'0')*10; buf++;}
	retval += (*buf-'0'); buf++;
	return(retval);
}


//reads decimal number from ascii bufer into integer. Specify length of number in digits parameter.
uint32_t bufGetDec32_t(Tbuf* buf, uint8_t digits)
{
	uint32_t retval=0;
	if(digits>=10) {retval += (*buf-'0')*1000000000; buf++;}
	if(digits>=9) {retval += (*buf-'0')*100000000; buf++;}
	if(digits>=8) {retval += (*buf-'0')*10000000; buf++;}
	if(digits>=7) {retval += (*buf-'0')*1000000; buf++;}
	if(digits>=6) {retval += (*buf-'0')*100000; buf++;}
	if(digits>=5) {retval += (*buf-'0')*10000; buf++;}
	if(digits>=4) {retval += (*buf-'0')*1000; buf++;}
	if(digits>=3) {retval += (*buf-'0')*100; buf++;}
	if(digits>=2) {retval += (*buf-'0')*10; buf++;}
	retval += (*buf-'0'); buf++;
	return(retval);
}


//reads hex number from ascii buffer into integer. Works for uint8_t and reads 2 indexes.
uint8_t bufGetHex8_t(Tbuf* buf)
{
	uint8_t retval=0;
	retval = bufHexToDec(*buf)&0x0F;
	buf++;
	retval = retval << 4;
	retval |= bufHexToDec(*buf)&0x0F;
	return(retval);
}


//reads hex number from ascii buffer into integer. Works for uint8_t and reads 4 indexes.
uint16_t bufGetHex16_t(Tbuf* buf)
{
	uint16_t retval;
	retval = bufGetHex8_t(buf);
	buf+=2;
	retval = retval << 8;
	retval |= bufGetHex8_t(buf);
	return(retval);
}


