#include "Board.h"
#include "common.h"
#include "dbgu.h"
#include "lcd_2x16.h"
#include "one_wire.h"
#include "ds18b20.h"
#include "spi.h"
#include "sd_spi.h"
#include "fat/fs_tools.h"


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//open debug channel
	debug_init_default();
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//init 1-Wire
	owInit();
	
	//initialize SPI interface
	spiInit();
	
	lcdInit();
	
	//initialize SD Card
	if(sdCardInit() == OK)
	{
		debug_msg("SD card initialized OK");
	}
	else
	{
		debug_msg("SD card initialization FAILED!");
		lcdLocate(0,0);
		lcdText("Card Init Fail");
		while(1);
	}
	
	//mount filesystem
	if(fsInit()!=FR_OK)
	{
		lcdLocate(0,0);
		lcdText("No Filesystem");
		while(1);
	}
	
	
}

#define FILE_NAME_LEN		(8+1+3+1) //8 chars name, 1 char dot, 3 chars extention, 1 char NULL

int main( void )
{
	init();
	
	lcdCls();
	lcdLocate(0,0);
	lcdText("SAM7XC Eval");
	lcdLocate(1,0);
	lcdText("Temp. Logger");
	delay_ms(1000);
	lcdCls();
	
	uint8_t rom[8];
	
	debug_msg("temperature logger");
	
	for(;;)
	{
		
		while(owReadRom(rom)!=OK)
		{
			lcdLocate(0,0);
			lcdText("Connect DS18B20!");
			delay_ms(100);
		}
		
		
		//rom is ok, read temperature
		ds18b20ConvertT();	//start temperature measurement
		delay_ms(1000);
		int16_t temperature = ds18b20ReadTempNoRom(); //get temperature from sensor
		
		//convert DS18B20 temperature register contents into readable text
		
		int8_t base = temperature >> 4;	//extract integer part
		
		uint8_t decimal = temperature & 0x0F;	//extract decimal part from bits 3...0
		decimal = (decimal * 10) >> 4;			//muliply decimal part by 10/16
		
		char logText[9] = "T=";
		if( temperature < 0 )
		{
			base = - base - 1;
			decimal = 10 - decimal;
			logText[2] = '-';
		}
		else
		{
			logText[2] = '+';
		}
		
		logText[3] = '0' + base / 100;
		base = base % 100;
		logText[4] = '0' + base / 10;
		base = base % 10;
		logText[5] = '0' + base;
		logText[6] = '.';
		logText[7] = '0' + decimal;
		logText[8] = 0;	//end of string
		
		lcdLocate(0,0); //set LCD cursor to upper line
		lcdText(logText); lcdText("  ");
		//debug_msg("text to write: "); debug_txt(logText);
		fsWriteLine("TEMPLOG.TXT",logText);
	}
	return(0);
}


