#include "common.h"
#include "adc.h"


/*
timings for MCK 48,054,857 Hz

PRESCAL setting:
	PRESCAL=4 => ADCClock=4,805,486 Hz => max for 10 bit resolution
	PRESCAL=2 => ADCClock=8,009,143 Hz => max for 8 bit resolution

for 10-bit resolution (PRESCAL=4)

STARTUP=0x0B => Startup Time=19,977 (20us is minimum startup time when returning from idle state)
use STARTUP=0x0C for safety ;-)

sample and hold time

for Zout=0 Sht = 589 ns
for Zout=1k Sht = 718 ns
for Zout=10k Sht = 1889 ns

SHTIM=4 => Sht = 1us
SHTIM=9 => Sht = 2,08us

*/


uint16_t adcGet(uint8_t channel)
{
	//define configuration fields in ADC_MR register
	const uint32_t SHTIM_MASK = 0x04 << 24;   //sample and hold time
	const uint32_t STARTUP_MASK = 0x0C << 16; //startup time
	const uint32_t PRESCAL_MASK = 0x4 << 8;   //prescaler setting
	
	//configure ADC
	AT91C_BASE_ADC->ADC_MR = SHTIM_MASK | STARTUP_MASK | PRESCAL_MASK;// | AT91C_ADC_TRGEN_EN;
	
	//activate requested channel
    channel = channel & 0x07;
    AT91C_BASE_ADC->ADC_CHER =	1<<channel;
	
	//start conversion!
	AT91C_BASE_ADC->ADC_CR = AT91C_ADC_START;
	
	//wait for adc DRDY bit set
	while( ! (AT91C_BASE_ADC->ADC_SR & AT91C_ADC_DRDY) );
	
	//read result from LCDR register
	uint16_t retval = AT91C_BASE_ADC->ADC_LCDR;
	
	//disable all channels
	AT91C_BASE_ADC->ADC_CHDR =	0xFF;
	
	return(retval);
}





