#include "common.h"
#include "lcd_2x16.h"
#include "adc.h"

void init(void)
{
	//enable peripheral clocks for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	debug_init_default();
	debug_msg("debug channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//initialize LCD
	lcdInit();
	lcdLocate(0,0);
	lcdText("ADC Basic");
	delay_ms(250);
	
	lcdLocate(1,0);
	lcdText("Value=");
	
}




int main( void )
{
	init();
	
	uint16_t adcResult;
	const uint8_t ADC_VR_CHANNEL = 4;
	
	for(;;)
	{
		adcResult = adcGet(ADC_VR_CHANNEL);
		
		//enter new result to LCD
		lcdLocate(1,6);
		lcdValue(adcResult); lcdText("  ");
		
		//display a bar
		lcdLocate(0,0);
		uint8_t i;
		adcResult = adcResult / 64 + 1; //fit result to 1-line-bar
		for(i=0;i<adcResult;i++) lcdChr('#');
		for(i=adcResult;i<16;i++) lcdChr('-');
		
		//a short, inaccurate delay
		delay_ms(100);
	}
	return(0);
}

