#include "common.h"
#include "lcd_2x16.h"
#include "adc.h"

void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	debug_init_default();
	debug_msg("debug channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	lcdInit();
	lcdLocate(0,0);
	lcdText("ADC Timer");
	lcdLocate(1,0);
	lcdText("Value=");
	
}




int main( void )
{
	init();
	
	//define channel to sample
	const uint8_t ADC_VR_CHANNEL = 4;
	
	//initialize sampling on defined channel
	adcSamplingInit(ADC_VR_CHANNEL);
	
	//set sampling rate here
	adcSetSamplingRate(8);
	//when setting faster sampling ragtes,
	//...don't forget to remove lcd calls from adcIrqHandler (adc.c)
	//because lcd accesses take long time to execute!
	
	for(;;)
	{
		LED_B_TOGGLE;
		delay_ms(100);
	}
	return(0);
}

