#include "common.h"
#include "lcd_2x16.h"
#include "pwm_bl.h"
#include "adc.h"

enum
{
	CONTROL_DIGITAL=0,
	CONTROL_ANALOG=1
};


void displaySettings(uint8_t ctrl, uint8_t backlite)
{
	lcdLocate(1,0);
	if(ctrl==CONTROL_DIGITAL)
		lcdText("SW1/Sw2: bl+/-");
	else
		lcdText("ADC 4: bl+/-  ");
	
	lcdLocate(0,0);
	lcdText("Backlite = ");
		lcdValue(backlite);
}


void init(void)
{
	//enable peripheral clocks for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	debug_init_default();
	debug_msg("debug channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//initialize LCD
	lcdInit();
	
	pwmblInit();
}


int main( void )
{
	init();
	
	uint8_t control = CONTROL_DIGITAL;
	uint8_t backlite = 0x40;
	const uint8_t ADC_VR_CHANNEL = 4;
	
	lcdLocate(0,0);
	lcdText("PWM = ");
	
	displaySettings(control,backlite);
	
	for(;;)
	{
		if(control == CONTROL_DIGITAL)
		{
			if(SW1_PRESSED)
			{
				backlite++;
				pwmblSet(backlite);
				displaySettings(control,backlite);
				delay_ms(5);
			}
			
			if(SW2_PRESSED)
			{
				backlite--;
				pwmblSet(backlite);
				displaySettings(control,backlite);
				delay_ms(5);
			}
		}
		else
		{
			backlite = adcGet(ADC_VR_CHANNEL) >> 2;
			displaySettings(control,backlite);
			pwmblSet(backlite);
		}
		
		//switch control mode: ADC versus pushbuttons
		if(SW3_PRESSED)
		{
			if(control == CONTROL_DIGITAL)
				control = CONTROL_ANALOG;
			else
				control = CONTROL_DIGITAL;
			
			displaySettings(control,backlite);
			
			while(SW3_PRESSED);
			delay_ms(100);
		}
		
		delay_ms(10);
	}
	return(0);
}

