#include "common.h"
#include "pwm_bl.h"

#define PWM_CHANNEL		2
#define PWM				AT91C_BASE_PWMC
#define PWM_ID			AT91C_ID_PWMC
#define PWM_PIO			AT91C_BASE_PIOB
#define PWM_PIN_MASK	AT91C_PIO_PB29
#define PWM_PERIPH		1		//0 for periph A
#define PWM_MAX_COUNT	255


void pwmblInit(void)
{
    //enable PWMC clock
    AT91C_BASE_PMC->PMC_PCER = 1<<PWM_ID;
	
	//disable PIO controller on PWM pin
	PWM_PIO->PIO_PDR = PWM_PIN_MASK;
	
	//select peripheral
	#if PWM_PERIPH
		PWM_PIO->PIO_BSR = PWM_PIN_MASK;
	#else
		PWM_PIO->PIO_ASR = PWM_PIN_MASK;
	#endif
	
	//define constant not included in AT91SAM7XCxxx.h
	const uint32_t AT91C_PWMC_CPRE_MCK32 = 0x05;
	
	//configure pwm channel
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CMR = AT91C_PWMC_CPOL | AT91C_PWMC_CPRE_MCK32;
	
	//set initial value of duty cycle
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CDTYR = 0x40;
	
	//set PWM period (max count)
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CPRDR = PWM_MAX_COUNT;
	
	PWM->PWMC_ENA = (1<<PWM_CHANNEL);
}

void pwmblSet(uint8_t bl)
{
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CUPDR = bl;
}

