#include "common.h"
#include "msc_bot.h"
#include "msc_usb.h"
#include "msc_ui.h"
#include "msc_if.h"


//Edit these macros to fit the code for your board and MCU
#define MSC_USB_PULLUP_PIO		AT91C_BASE_PIOA
#define MSC_USB_PULLUP_MASK		AT91C_PIO_PA9
#define MSC_USB_VDET_PIO		AT91C_BASE_PIOA
#define MSC_USB_VDET_MASK		AT91C_PIO_PA8


//mscMaxLUN is placed in msc_usb.h
extern volatile uint8_t mscMaxLUN;


//called once, initializes USB Device and PIO pins
uint8_t mscInit(uint8_t maxLun)
{
    // Set the PLL USB Divider
    AT91C_BASE_CKGR->CKGR_PLLR |= AT91C_CKGR_USBDIV_1 ;
	
	//pullup pin
    MSC_USB_PULLUP_PIO->PIO_PER = MSC_USB_PULLUP_MASK;
    MSC_USB_PULLUP_PIO->PIO_OER = MSC_USB_PULLUP_MASK;
	
	//vdetect pin
	MSC_USB_VDET_PIO->PIO_PER = MSC_USB_VDET_MASK;
	MSC_USB_VDET_PIO->PIO_ODR = MSC_USB_VDET_MASK;
	MSC_USB_VDET_PIO->PIO_PPUDR = MSC_USB_VDET_MASK;
	
	//mscMaxLUN is in msc_usb.h
	mscMaxLUN=maxLun;
	
	return(1);
}



//returns 1 if USB is connected to the host
inline uint8_t mscPlugInserted(void)
{
	if(MSC_USB_VDET_PIO->PIO_PDSR&MSC_USB_VDET_MASK) return(1); else return(0);
}




//may be called multiple times. Reinitializes USB Device and starts disk or card.
uint8_t mscStart(void)
{
    AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_UDP;
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_UDP);
	
    USB_EP2_CurrentBank = AT91C_UDP_RX_DATA_BK0;
	
    MSC_USB_PULLUP_PIO->PIO_SODR = MSC_USB_PULLUP_MASK;
	
	return(1);
}


//may be called multiple times. Stops disk or disables card, disables USB pullup pin
uint8_t mscStop(void)
{
    MSC_USB_PULLUP_PIO->PIO_CODR = MSC_USB_PULLUP_MASK;
    AT91C_BASE_PMC->PMC_SCDR = AT91C_PMC_UDP;
    AT91C_BASE_PMC->PMC_PCDR = (1 << AT91C_ID_UDP);
	return(1);
}


//call it in a loop or a task
inline void mscUIEnumerate(void)
{
	mscEnumerate();
}
