#include "Board.h"
#include "common.h"
#include "dbgu.h"
#include "spi.h"
#include "sd_spi.h"
#include "mass_storage/msc_ui.h"
#include "aes.h"



void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//open debug channel
	debug_init_default();
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//initialize SPI interface
	spiInit();
	
	//initialize SD Card
	if(sdCardInit() == OK)
	{
		debug_msg("SD card initialized OK");
	}
	else
	{
		debug_msg("SD card initialization FAILED!");
	}
	
	//initialize Mass Storage Device stuff
	mscInit(0);
	debug_msg("USB mass storage class initialized");
	
	aesInit();
	
	const uint32_t key[4] = {0xAABBCCDD, 0xCAFEDEDE, 0x0000BEEF, 0x11223344};
	
	aesSetKey((uint32_t*)key);
}



int main( void )
{
	init();
	
	for(;;)
	{
		while(!mscPlugInserted());
		debug_msg("USB plug inserted");
		
		//start USB transmission and reinitialize mass storage class code
		mscStart();
		debug_msg("usb transmission started");
		
		while(mscPlugInserted())
		{
			mscUIEnumerate();
		}
		
		debug_msg("USB Plug pulled out");
		
		//stop USB transmission
		mscStop();
		debug_msg("USB transmission terminated");
		
	}//for(;;)
	return(0);
}


