#include "common.h"
#include "lcd_2x16.h"

//#define LCD_DELAY_MS(a)	delay_ms(a)	//FreeRTOS task delay
#define LCD_DELAY_MS(a)	delay_ms(a)	//dead loop delay

#define LCD_PIO			AT91C_BASE_PIOA

#define LCD_RS_MASK		AT91C_PIO_PA19
#define LCD_ENA_MASK	AT91C_PIO_PA21
#define LCD_DB4_MASK	AT91C_PIO_PA22
#define LCD_DB5_MASK	AT91C_PIO_PA23
#define LCD_DB6_MASK	AT91C_PIO_PA24
#define LCD_DB7_MASK	AT91C_PIO_PA25

#define LCD_PINS_MASK	(LCD_RS_MASK | LCD_ENA_MASK | LCD_DB4_MASK | LCD_DB5_MASK | LCD_DB6_MASK | LCD_DB7_MASK)

#define LCD_PINS_INIT	{LCD_PIO->PIO_PER = LCD_PINS_MASK; LCD_PIO->PIO_MDER = LCD_PINS_MASK; LCD_PIO->PIO_OER = LCD_PINS_MASK;}

#define LCD_ENA_1		LCD_PIO->PIO_SODR = LCD_ENA_MASK
#define LCD_ENA_0		LCD_PIO->PIO_CODR = LCD_ENA_MASK
#define LCD_RS_1		LCD_PIO->PIO_SODR = LCD_RS_MASK
#define LCD_RS_0		LCD_PIO->PIO_CODR = LCD_RS_MASK
#define LCD_DB4_1		LCD_PIO->PIO_SODR = LCD_DB4_MASK
#define LCD_DB4_0		LCD_PIO->PIO_CODR = LCD_DB4_MASK
#define LCD_DB5_1		LCD_PIO->PIO_SODR = LCD_DB5_MASK
#define LCD_DB5_0		LCD_PIO->PIO_CODR = LCD_DB5_MASK
#define LCD_DB6_1		LCD_PIO->PIO_SODR = LCD_DB6_MASK
#define LCD_DB6_0		LCD_PIO->PIO_CODR = LCD_DB6_MASK
#define LCD_DB7_1		LCD_PIO->PIO_SODR = LCD_DB7_MASK
#define LCD_DB7_0		LCD_PIO->PIO_CODR = LCD_DB7_MASK


static inline void lcdHalfByte(uint8_t value);
static inline void lcdFullByte(uint8_t value);
static void lcdCommand(uint8_t cmd);
static uint8_t dec2hex(uint8_t val);


static inline void lcdHalfByte(uint8_t value)
{
    if(value&(1<<3)) LCD_DB7_1; else LCD_DB7_0;
    if(value&(1<<2)) LCD_DB6_1; else LCD_DB6_0;
    if(value&(1<<1)) LCD_DB5_1; else LCD_DB5_0;
    if(value&(1<<0)) LCD_DB4_1; else LCD_DB4_0;
	LCD_DELAY_MS(1);
}


static inline void lcdFullByte(uint8_t value)
{
    LCD_ENA_1;
    //upper half byte
    lcdHalfByte(value>>4);
    LCD_ENA_0;
	LCD_DELAY_MS(1);
    LCD_ENA_1;
    lcdHalfByte(value);
    LCD_ENA_0;
	//for the lowest possible power consumption:
	LCD_RS_1;
    LCD_ENA_1;
	lcdHalfByte(0x0F);
}


static void lcdCommand(uint8_t cmd)
{
	LCD_RS_0;
	lcdFullByte(cmd);
	LCD_DELAY_MS(1);
}


void lcdChr(uint8_t chr)
{
	LCD_RS_1;
	lcdFullByte(chr);
}


void lcdCls(void)
{
	lcdCommand(0x01);
	LCD_DELAY_MS(2);
}


void lcdText(const char *ptr)
{
	while(*ptr) lcdChr(*ptr++);
}


void lcdLocate(uint8_t lcd_y, uint8_t lcd_x)
{
	lcdCommand((lcd_y*0x40+lcd_x)|0x80);
}


static uint8_t dec2hex(uint8_t val)
{
	val &= 0x0F;
	if(val<10) val+=0x30;
	else val=val-10+'A';
	return(val);
}


void lcdHex(uint8_t val)
{
	lcdChr(dec2hex(val>>4));
	lcdChr(dec2hex(val&0x0F));
}


void lcdValue(uint32_t value)
{
    if(value>0x00FFFFFF) lcdHex(value>>(3*8));
    if(value>0x0000FFFF) lcdHex(value>>(2*8));
    if(value>0x000000FF) lcdHex(value>>(1*8));
    lcdHex(value>>(0*8));
}


void lcdByte(uint8_t value)
{
	if(value>99) lcdChr(value/100+0x30);
	if(value>9) lcdChr((value%100)/10+0x30);
	lcdChr((value%100)%10+0x30);
}



void lcdInit(void)
{
	LCD_PINS_INIT;
	
	uint8_t i;
	LCD_RS_0;
	LCD_DELAY_MS(10);
	
	for(i=0;i<3;i++)
	{
		LCD_ENA_1;
        lcdHalfByte(0x03);
		LCD_ENA_0;
		LCD_DELAY_MS(5);
	}
	LCD_ENA_1;
    lcdHalfByte(0x02);
	LCD_ENA_0;
	LCD_DELAY_MS(1);
	
	lcdCommand(0x28);
	lcdCommand(0x08);
	lcdCls();
	lcdCommand(0x06);
	lcdCommand(0x0C);
}


