#include "Board.h"
#include "common.h"
#include "lcd_2x16.h"
#include "one_wire.h"
#include "ds18b20.h"


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//initialize LCD
	lcdInit();
	
	//initialize 1-Wire bus
	owInit();
}




int main( void )
{
	init();
	
	lcdCls();
	lcdLocate(0,0);
	lcdText("SAM7XC Eval");
	lcdLocate(1,0);
	lcdText("Thermometer");
	delay_ms(500);
	lcdCls();
	
	uint8_t rom[8];
	
	
	
	for(;;)
	{
		//try to read ROM memory of dallas thermometer
		if(owReadRom(rom)==OK)
		{
			LED_R_OFF;
			LED_G_ON;
			
			lcdLocate(1,0);	//set LCD cursor to lower line
			uint8_t i;
			for(i=0;i<8;i++)
			{
				lcdHex(rom[i]);
			}
			
			//rom is ok, read temperature
			ds18b20ConvertT();	//start temperature measurement
			delay_ms(1000);
			int16_t temperature = ds18b20ReadTempNoRom(); //get temperature from sensor
			
			lcdLocate(0,0); //set LCD cursor to upper line
			
			//convert DS18B20 temperature register contents into readable text on LCD
			
			int8_t base = temperature >> 4;	//extract integer part
			
			uint8_t decimal = temperature & 0x0F;	//extract decimal part from bits 3...0
			decimal = (decimal * 10) >> 4;			//muliply decimal part by 10/16
			
			if( temperature < 0 )
			{
				base = - base - 1;
				decimal = 10 - decimal;
				lcdChr('-');
			}
			else
			{
				lcdChr('+');
			}
			
			lcdByte(base);
			lcdChr('.');
			lcdByte(decimal);
			lcdChr(' ');
		}//if(owReadRom(rom)==OK)
		else
		{
			LED_R_ON;
			LED_G_OFF;
		}
	}
	return(0);
}


