#include "Board.h"
#include "common.h"
#include "dbgu.h"
#include "lcd_2x16.h"


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//open debug channel
	debug_init_default();
	debug_msg("MSG Debug UART channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//initialize LCD
	lcdInit();
	
}


void debugTest(void)
{
	#define TEST_ARRAY_SIZE   10
	uint32_t testArray[TEST_ARRAY_SIZE] =
	{
		0x01234567, 0xAABBCCDD,
		0x11223344, 0x00000001,
		0x00000002, 0x00000003,
		0x00000004, 0x00000005,
		0x00000006, 0x00000007
	};
	
	debug_value("testArray[2] = ",testArray[2]);
	
	debug_msg("negative integer -123: "); debug_int(-123);
	
	debug_msg("testArray contents (byte):");
	debug_array(testArray, TEST_ARRAY_SIZE*sizeof(uint32_t) );
	
	debug_msg("testArray memory dump:");
	debug_dump(testArray, TEST_ARRAY_SIZE*sizeof(uint32_t) );
	
	debug_msg("DBGU Register dump:");
	debug_dump(AT91C_BASE_DBGU,0x48);
	
	debug_menu();
	
	debug_msg("enter a text (max 20 chars): ");
	char inputText[20];
	debug_input_string(inputText, 20);
	debug_msg("you entered: ");
		debug_txt(inputText);
	
	debug_msg("enter a 32-bit number in hex, ESC to use default number (0): ");
	uint32_t inputNumber;
	inputNumber = debug_input_hex(0);
	debug_value("you entered ",inputNumber);
	
	
	debug_msg("Debug test finished! Press any key to continue...");
	debug_waitkey();
}




int main( void )
{
	init();
	
	lcdCls();
	lcdLocate(0,0);
	lcdText("SAM7XC Eval");
	lcdLocate(1,0);
	lcdText("Debug UART test");
	
	debugTest();
	
	for(;;)
	{
		
	}
	return(0);
}


