#include "common.h"
#include "lcd_2x16.h"
#include "usart_dma_irq.h"


#define RX_BUFFER_LEN		40
uint8_t rxBuffer[RX_BUFFER_LEN];
uint8_t dmaBuffer[RX_BUFFER_LEN];


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	debug_init_default();
	debug_msg("debug channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;

	lcdInit();
	
	usartDmaIrqInit( dmaBuffer , RX_BUFFER_LEN );
}




int main( void )
{
	init();
	for(;;)
	{
		if(SW1_PRESSED)
		{
			debug_msg("sw1 pressed!");
			lcdLocate(1,0); lcdText("sw1 pressed!    ");
			const char * TEXT_TO_SEND = "Text to send thru USART DMA TX channel!";
			usartDmaIrqTransmit((uint8_t*)TEXT_TO_SEND,strlen(TEXT_TO_SEND));
			delay_ms(200);
		}
		
		//delay_ms(250);
		
		uint16_t len;
		len = usartDmaIrqReceive( rxBuffer );
		if(len)
		{
			LED_Y_ON;
			//display length of incoming data
			debug_value("Received new data! length =", len);
			lcdLocate(0,0); lcdText("Rxed, len="); lcdHex(len); lcdText("    ");
			
			//display buffer contents
			debug_msg("### contents: ");
			debug_txt_limit( (char*) rxBuffer, len );
			lcdLocate(1,0);
			uint8_t i;
			if(len > 16) len = 16;
			for(i=0;i<len;i++) lcdChr(rxBuffer[i]); //print up to the first 16 chars on lcd
			for(i=len;i<16;i++) lcdChr(' '); //print spaces until end of lcd line
			
			delay_ms(250);
			
			LED_Y_OFF;
		}
	}
	return(0);
}
