#include "common.h"
#include "spi.h"
#include "lcd/lcd3310.h"


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//open debug channel
	debug_init_default();
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	spiInit();
	
	lcdInit();
}






int main( void )
{
	init();
	
	lcdSetFont(FONT_NORMAL);
	lcd("3310 Display",0,0);
	lcdSetFont(FONT_BOLD);
	lcd("TEST!",20,20);
	lcd("Press joystick",5,LCD_Y_SIZE-10);
	lcdRefresh();
	
	while( !( JOY_ENTER_PRESS || JOY_LEFT_PRESS || JOY_RIGHT_PRESS || JOY_UP_PRESS || JOY_DOWN_PRESS ) )
	{
		lcdInvertArea(0,LCD_X_SIZE,LCD_Y_SIZE-11,LCD_Y_SIZE-1);
		delay_ms(500);
		lcdRefresh();
	}
	
	lcdClear();
	lcdTPixel x=LCD_X_SIZE/2, y=LCD_Y_SIZE/2;
	lcdPixelSet(x,y);
	lcdRefresh();
	
	for(;;)
	{
		if(JOY_ENTER_PRESS)
		{
			lcdClear();
			x=LCD_X_SIZE/2;
			y=LCD_Y_SIZE/2;
			lcdPixelSet(x,y);
			lcdRefresh();
			delay_ms(100);
		}
		if(JOY_LEFT_PRESS)
		{
			if(x > 0) x--;
			lcdPixelSet(x,y);
			lcdRefresh();
			delay_ms(10);
		}
		if(JOY_RIGHT_PRESS)
		{
			if(x < LCD_X_SIZE - 1) x++;
			lcdPixelSet(x,y);
			lcdRefresh();
			delay_ms(10);
		}
		if(JOY_DOWN_PRESS)
		{
			if(y > 0) y--;
			lcdPixelSet(x,y);
			lcdRefresh();
			delay_ms(10);
		}
		if(JOY_UP_PRESS)
		{
			if(y < LCD_Y_SIZE - 1) y++;
			lcdPixelSet(x,y);
			lcdRefresh();
			delay_ms(10);
		}
	}
	
	return(0);
}


