#include "common.h"
#include "sd_spi.h"



uint8_t sdCardInit(void)
{
	int16_t i;
	uint8_t resp;
    
    sdSPIInit();
		
	/* Try to send reset command up to 100 times */
	i=100;
	do {
		sdCommand(0, 0, 0);
		resp=sdResponse8_t();
	}
	while(resp!=1 && i--);
	
	if(resp!=1)
    {
		//DBG((TXT("CMD0 fail.\n")));
		if(resp==0xff)
        {
			return(0);
		}
		else
        {
			//sd_Resp8bError(iface,resp);
			return(0);
		}
	}

	/* Wait till card is ready initialising (returns 0 on CMD1) */
	/* Try up to 32000 times. */
	i=32000;
	do{
		sdCommand(1, 0, 0);
		
		resp=sdResponse8_t();
		if(resp!=0)
        {
            
        }
			//sd_Resp8bError(iface,resp);
	}
	while(resp==1 && i--);
	
	if(resp!=0)
    {
		//sd_Resp8bError(iface,resp);
		return(0);
	}
	
	return(1);
}




uint8_t sdSPIInit(void)
{
    //NPCS is still controlled by PIO:
    SD_NPCS_PIO->PIO_PER=SD_NPCS_PIN_MASK;
    SD_NPCS_PIO->PIO_SODR=SD_NPCS_PIN_MASK;
    SD_NPCS_PIO->PIO_OER=SD_NPCS_PIN_MASK;
    
    //SD SPI Settings
    SD_SPI->SPI_CSR[SD_NPCS_NUMBER] = AT91C_SPI_CPOL | AT91C_SPI_BITS_8;
    //set low speed for init
    sdSPISetSpeed(0xFE);
    
    //send 20 spi commands for init
    uint8_t i;
	for(i=0;i<21;i++)
    {
		sdSPIDataTransfer(0xFF);
	}
    
    //from now on SD NPCS pin may be controlled by SPI harware
    SD_NPCS_PIO->PIO_PDR=SD_NPCS_PIN_MASK;
    SD_NPCS_PIO->PIO_CODR=SD_NPCS_PIN_MASK;
    SD_NPCS_PIO->PIO_ODR=SD_NPCS_PIN_MASK;
    AT91F_PIO_SelectPeripheral(SD_NPCS_PIO,SD_NPCS_PIN_MASK,SD_NPCS_PIO_PERIPH);
    
    sdSPISetSpeed(SPI_SCBR_MIN);
    
    return(1);
}


inline void AT91F_PIO_SelectPeripheral(AT91PS_PIO pPio, uint32_t Mask, uint8_t PeriphSelect)
{
    if(PeriphSelect==0)
    {
        pPio->PIO_ASR=Mask;
    }
    else
    {
        pPio->PIO_BSR=Mask;
    }
}


void sdSPISetSpeed(uint8_t speed)
{
	uint32_t reg;

	if ( speed < SPI_SCBR_MIN ) speed = SPI_SCBR_MIN;
	if ( speed > 1 ) speed &= 0xFE;

	reg = SD_SPI->SPI_CSR[SD_NPCS_NUMBER];                          //copy
	reg = ( reg & ~(AT91C_SPI_SCBR) ) | ( (uint32_t)speed << 8 );   //modify
	SD_SPI->SPI_CSR[SD_NPCS_NUMBER] = reg;                          //write modified
}


uint8_t sdSPIDataTransfer(uint8_t dat)
{
    /*uint8_t RXedData;
	
	while( !( SD_SPI->SPI_SR & AT91C_SPI_TDRE ) ); // transfer compl. wait
	SD_SPI->SPI_TDR = ( (uint16_t)(dat) | ((uint32_t)(SD_NPCS_SELECT)<<16) );

	while( !( SD_SPI->SPI_SR & AT91C_SPI_RDRF ) ); // wait for char
	RXedData = (uint8_t)( SD_SPI->SPI_RDR );

	return RXedData;*/
	
	return(spiDataTransfer(dat));
	
}


uint8_t sdCommand(uint8_t cmd, uint16_t paramx, uint16_t paramy)
{
	sdSPIDataTransfer(0xff);
	sdSPIDataTransfer(0x40 | cmd);
	sdSPIDataTransfer((uint8_t) (paramx >> 8)); /* MSB of parameter x */
	sdSPIDataTransfer((uint8_t) (paramx)); /* LSB of parameter x */
	sdSPIDataTransfer((uint8_t) (paramy >> 8)); /* MSB of parameter y */
	sdSPIDataTransfer((uint8_t) (paramy)); /* LSB of parameter y */
	sdSPIDataTransfer(0x95); /* Checksum (should be only valid for first command (0) */
	sdSPIDataTransfer(0xff); /* eat empty command - response */
    return(1);
}


uint8_t sdResponse8_t(void)
{
	uint8_t i;
	uint8_t resp;
	
	/* Respone will come after 1 - 8 pings */
	for ( i=0; i<8; i++ ) {
		resp = sdSPIDataTransfer(0xff);
		if(resp != 0xff)
			return(resp);
	}
		
	return(resp);
}


uint16_t sdResponse16_t(void)
{
	uint16_t resp;
	
	resp = ( sdResponse8_t() << 8 ) & 0xff00;
	resp |= sdSPIDataTransfer(0xff);
	
	return(resp);
}


uint8_t sdGetSizeInfo(uint32_t *NumberOfBlocks, uint16_t *BlockSize)
{
	uint8_t cardresp, i, by;
	uint8_t iob[16];
	uint16_t c_size, c_size_mult;//read_bl_len;
	
	sdCommand(CMDREADCSD, 0, 0);
	
	do {
		cardresp =sdResponse8_t();
	} while ( cardresp != 0xFE );
	
	for( i=0; i<16; i++)
    {
		iob[i] = sdSPIDataTransfer(0xff);
	}
	
	sdSPIDataTransfer(0xff);
	sdSPIDataTransfer(0xff);
	
	c_size = iob[6] & 0x03; // bits 1..0
	c_size <<= 10;
	c_size += (uint16_t)iob[7]<<2;
	c_size += iob[8]>>6; //we've got c_size
	
	by= iob[5] & 0x0F;
	
	by=iob[9] & 0x03;
	by <<= 1;
	by += iob[10] >> 7;
	
	c_size_mult = 1;
	c_size_mult <<= (2+by); //now we've got c_size_mult
	
    //return values to variables
    *BlockSize=512; //copy to output
    *NumberOfBlocks=c_size_mult*(c_size+1)-1;
    
    return(1);
}


uint8_t sdReadBlock(uint32_t BlockAddress, uint8_t *Buffer)
{
	LED_Y_ON;
	
	uint8_t cardresp;
	uint8_t firstblock;
	uint8_t c;
	uint16_t fb_timeout=0xffff;
	uint32_t i;
	uint32_t place;
	
	place=512*BlockAddress;
	sdCommand(CMDREAD, (uint16_t) (place >> 16), (uint16_t) place);
	
	cardresp=sdResponse8_t();
	
	do
		firstblock=sdResponse8_t(); 
	while(firstblock==0xff && fb_timeout--);
	
	if(cardresp!=0x00 || firstblock!=0xfe)
    {
		LED_Y_OFF;
		return(0);
	}
	
	for(i=0;i<512;i++)
    {
		c = sdSPIDataTransfer(0xff);
        Buffer[i] = c;
	}
	
	//ignore checksum
	sdSPIDataTransfer(0xff);
	sdSPIDataTransfer(0xff);

	LED_Y_OFF;
	return(1);

}


uint8_t sdWriteBlock(uint32_t BlockAddress, uint8_t *Buffer)
{
	LED_R_ON;
	uint32_t place;
	uint16_t i;
	
	place=512*BlockAddress;
	sdCommand(CMDWRITE, (uint16_t) (place >> 16), (uint16_t) place);
	
	sdResponse8_t();
	
	sdSPIDataTransfer(0xfe);
	for(i=0;i<512;i++)
    {
        sdSPIDataTransfer(Buffer[i]);
    }
	sdSPIDataTransfer(0xff);	//checksum
	sdSPIDataTransfer(0xff);	//checksum
	sdSPIDataTransfer(0xff);
	while(sdSPIDataTransfer(0xff)!=0xff);	//wait for response
	
	LED_R_OFF;
	return(1);
}


uint8_t sdReadBlocks(uint32_t LBA, uint8_t blocks, uint8_t *buf)
{
	uint8_t i;
	for(i=0;i<blocks;i++)
	{
		sdReadBlock(LBA+i,buf);
		buf+=512;
	}
	return(OK);
}


uint8_t sdWriteBlocks(uint32_t LBA, uint8_t blocks, uint8_t *buf)
{
	uint8_t i;
	for(i=0;i<blocks;i++)
	{
		sdWriteBlock(LBA+i,buf);
		buf+=512;
	}
	return(OK);
}


